==> fetch.php?media=uls%3Auls_update-1.9.5.tgz: update/von_1.9.4/sql/genreg.sql <==
set @@character_set_client = 'utf8';

delimiter //
DROP FUNCTION IF EXISTS genreg//
create function genreg(r varchar(2000)) returns varchar(2000)
  deterministic
  sql security invoker
  begin
    declare rx, rp, rr, rs, re varchar(2000);
    declare b1, b2, b3, ls, le, i integer;

    if length(r) = 0
     then
      return '.*';
    end if;
    if r = '␣' OR r = ' ' then
      return '^ $';
    end if;
    set rx = replace(replace(replace(replace(replace(replace(replace(replace(replace(trim(r), '\\{', 0x01), '\\}', 0x02), '\\', '\\\\'), '.', '\\.'), '*', '.*'), '.*.*', '\\*'), '?', '.'), '.*.', '\\?'), '^', '\\^');
    set rx = replace(replace(replace(replace(replace(replace(replace(replace(replace(rx, '^', '\\^'), '$', '\\$'), '[', '\\['), ']', '\\]'), '+', '\\+'), '(', '\\('), ')', '\\)'), '|', '\\|'), ',', '$|^');

    set b1 = regexp_instr(rx, '{[0-9]+-[0-9]+}');
    while b1 > 0 do
      set rp = '';
      set b2 = locate('-', rx, b1);
      set b3 = locate('}', rx, b1);
      set rs = trim(substr(rx, b1+1, b2 - b1 - 1));
      set ls = length(rs);
      set re = trim(substr(rx, b2+1, b3 - b2 - 1));
      set le = length(re);
      if 0+rs <= 0+re then
        if ls = le then
          set i = 1;
          while i <= ls and left(rs, i) = left(re, i) do
            set i = i + 1;
          end while;
          if i > 1 then
            set rp = left(rs, i-1);
            set ls = ls - i + 1;
            set le = ls;
            set rs = substr(rs, i);
            set re = substr(re, i);
          end if;
        end if;
        if le > 1 then
          if right(rs, 1) = '9' then
            set rr = rs;
          else
            set rr = concat(left(rs, ls - 1), '[', right(rs, 1), '-9]');
          end if;
        else
          set rr = concat(left(rs, ls - 1), '[', right(rs, 1), '-', right(re, 1), ']');
        end if;
        set i = 2;
        while i < le and i <= ls do
          if substr(rs, -i, 1) = '9' then
            set rr = concat(rr, '|', left(rs, ls - i + 1), repeat('[0-9]', i - 1));
          else
            set rr = concat(rr, '|', left(rs, ls - i), '[', substr(rs, ls - i + 1, 1)+1, '-9]', repeat('[0-9]', i - 1));
          end if;
          set i = i + 1;
        end while;
        while i < le do
          set rr = concat(rr, '|[1-9]', repeat('[0-9]', i - 1));
          set i = i + 1;
        end while;
        if le > 1 then
          if le = ls then
            if left(rs, 1) + 2 < left(re, 1) then
              set rr = concat(rr, '|[', left(rs, 1)+1, '-', left(re, 1)-1, ']', repeat('[0-9]', le - 1));
            elseif left(rs, 1) + 2 = left(re, 1) then
              set rr = concat(rr, '|', left(rs, 1)+1, repeat('[0-9]', le-1));
            end if;
          else
            if left(re, 1) > 2 then
              set rr = concat(rr, '|[1-', left(re, 1)-1, ']', repeat('[0-9]', le-1));
            elseif left(re, 1) = 2 then
              set rr = concat(rr, '|1', repeat('[0-9]', le-1));
            end if;
          end if;
          set i = 1;
          while i < le - 1 do
            if substr(re, i + 1, 1) = '1' then
              set rr = concat(rr, '|', left(re, i), '0', repeat('[0-9]', le - i - 1));
            elseif substr(re, i + 1, 1) > 1 then
              set rr = concat(rr, '|', left(re, i), '[0-', substr(re, i + 1, 1) - 1, ']', repeat('[0-9]', le - i - 1));
            end if;
            set i = i + 1;
          end while;
          if substr(re, i + 1, 1) = '0' then
            set rr = concat(rr, '|', left(re, i), '0');
          else
            set rr = concat(rr, '|', left(re, i), '[0-', substr(re, i + 1, 1), ']');
          end if;
        end if;
        if rr like '%|%' then
          set rx = concat(left(rx, b1 - 1), rp, '(', rr, ')', substr(rx, b3 + 1));
        else
          set rx = concat(left(rx, b1 - 1), rp, rr, substr(rx, b3 + 1));
        end if;
      else
        set rx = concat(left(rx, b1 - 1), 0x03, substr(rx, b1 + 1));
      end if;
      set b1 = regexp_instr(rx, '{[0-9]+-[0-9]+}');
    end while;

    return concat('^', replace(replace(replace(rx, 0x01, '\\{'), 0x02, '\\}'), 0x03, '{'), '$');
  end//

GRANT EXECUTE on FUNCTION genreg
  to logginglesen@'localhost', loggingadmin@'localhost', logging@'localhost'//

==> fetch.php?media=uls%3Auls_update-1.9.6.tgz: update/von_1.9.5/sql/genreg.sql <==
-- genreg.sql
--
-- function genreg(r) returns varchar
-- r: ULS-Wildcard
-- return Regexp
--
-- ULS-Wildcard         Regexp
--          ,           $|^
--          *            .*
--          ?            .
--         \?           \?
--         \*           \*
--         \,            ,
--         \\           \\
--         \{           \{
--         \}           \}
--         \!            !          (am Zeilenanfang)
--  ( ) [ ] . + $ ^ |   \<c>
--  {<s1>[<sn>..]-[<e1>[<en>..]}  (rangeregexp)
--
-- Sample r = "a*b?c{07-16}" -> "^a.*b.c(0[7-9]|1[0-6])$"



set @@character_set_client = 'utf8';

delimiter //
DROP FUNCTION IF EXISTS genreg//
create function genreg(r varchar(2000)) returns varchar(2000)
  deterministic
  sql security invoker
  begin
    declare rx, rp, rr, rs, re varchar(2000);
    declare b1, b2, b3, ls, le, i integer;

    if length(r) = 0
     then
      return '.*';
    end if;
    if r = '␣' OR r = ' ' then
      return '^ $';
    end if;
    set rx = replace(replace(replace(replace(replace(replace(replace(replace(replace(trim(r), '\\\\', 0x01), '\\{', 0x02), '\\}', 0x03), '\\,', 0x04), '\\?', 0x05), '\\*', 0x06), '.', '\\.'), '*', '.*'), '?', '.');
    set rx = replace(replace(replace(replace(replace(replace(replace(replace(replace(replace(rx, '\\!', '!'), '^', '\\^'), '$', '\\$'), '[', '\\['), ']', '\\]'), '+', '\\+'), '(', '\\('), ')', '\\)'), '|', '\\|'), ',', '$|^');

    set b1 = regexp_instr(rx, '{[0-9]+-[0-9]+}');
    while b1 > 0 do
      set rp = '';
      set b2 = locate('-', rx, b1);
      set b3 = locate('}', rx, b1);
      set rs = trim(substr(rx, b1+1, b2 - b1 - 1));
      set ls = length(rs);
      set re = trim(substr(rx, b2+1, b3 - b2 - 1));
      set le = length(re);
      if 0+rs <= 0+re then
        if ls = le then
          set i = 1;
          while i <= ls and left(rs, i) = left(re, i) do
            set i = i + 1;
          end while;
          if i > 1 then
            set rp = left(rs, i-1);
            set ls = ls - i + 1;
            set le = ls;
            set rs = substr(rs, i);
            set re = substr(re, i);
          end if;
        end if;
        if le > 1 then
          if right(rs, 1) = '9' then
            set rr = rs;
          else
            set rr = concat(left(rs, ls - 1), '[', right(rs, 1), '-9]');
          end if;
        else
          set rr = concat(left(rs, ls - 1), '[', right(rs, 1), '-', right(re, 1), ']');
        end if;
        set i = 2;
        while i < le and i <= ls do
          if substr(rs, -i, 1) = '9' then
            set rr = concat(rr, '|', left(rs, ls - i + 1), repeat('[0-9]', i - 1));
          else
            set rr = concat(rr, '|', left(rs, ls - i), '[', substr(rs, ls - i + 1, 1)+1, '-9]', repeat('[0-9]', i - 1));
          end if;
          set i = i + 1;
        end while;
        while i < le do
          set rr = concat(rr, '|[1-9]', repeat('[0-9]', i - 1));
          set i = i + 1;
        end while;
        if le > 1 then
          if le = ls then
            if left(rs, 1) + 2 < left(re, 1) then
              set rr = concat(rr, '|[', left(rs, 1)+1, '-', left(re, 1)-1, ']', repeat('[0-9]', le - 1));
            elseif left(rs, 1) + 2 = left(re, 1) then
              set rr = concat(rr, '|', left(rs, 1)+1, repeat('[0-9]', le-1));
            end if;
          else
            if left(re, 1) > 2 then
              set rr = concat(rr, '|[1-', left(re, 1)-1, ']', repeat('[0-9]', le-1));
            elseif left(re, 1) = 2 then
              set rr = concat(rr, '|1', repeat('[0-9]', le-1));
            end if;
          end if;
          set i = 1;
          while i < le - 1 do
            if substr(re, i + 1, 1) = '1' then
              set rr = concat(rr, '|', left(re, i), '0', repeat('[0-9]', le - i - 1));
            elseif substr(re, i + 1, 1) > 1 then
              set rr = concat(rr, '|', left(re, i), '[0-', substr(re, i + 1, 1) - 1, ']', repeat('[0-9]', le - i - 1));
            end if;
            set i = i + 1;
          end while;
          if substr(re, i + 1, 1) = '0' then
            set rr = concat(rr, '|', left(re, i), '0');
          else
            set rr = concat(rr, '|', left(re, i), '[0-', substr(re, i + 1, 1), ']');
          end if;
        end if;
        if rr like '%|%' then
          set rx = concat(left(rx, b1 - 1), rp, '(', rr, ')', substr(rx, b3 + 1));
        else
          set rx = concat(left(rx, b1 - 1), rp, rr, substr(rx, b3 + 1));
        end if;
      else
        set rx = concat(left(rx, b1 - 1), 0x02, substr(rx, b1 + 1));
      end if;
      set b1 = regexp_instr(rx, '{[0-9]+-[0-9]+}');
    end while;

    return concat('^', replace(replace(replace(replace(replace(replace(rx, 0x01, '\\\\'), 0x02, '\\{'), 0x03, '\\}'), 0x04, ','), 0x05, '\\?'), 0x06, '\\*'), '$');
  end//

GRANT EXECUTE on FUNCTION genreg
  to logginglesen@'localhost', loggingadmin@'localhost', logging@'localhost'//
